@@ -386,7 +386,7 @@ QList<Plasma::Types::Location> Corona::freeEdges(int screen) const
 
     const auto containments = this->containments();
     for (Containment *containment : containments) {
-        if (containment->screen() == screen && freeEdges.contains(containment->location())) {
+        if (containment->lastScreen() == screen && freeEdges.contains(containment->location())) {
             freeEdges.removeAll(containment->location());
         }
     }
@@ -447,7 +447,10 @@ CoronaPrivate::CoronaPrivate(Corona *corona)
 
 CoronaPrivate::~CoronaPrivate()
 {
-    qDeleteAll(containments);
+    // Do not qDeleteAll. The list gets mutated as objects are destroyed because we are connected to the destroyed signal!
+    while (!containments.isEmpty()) {
+        delete containments.takeAt(0);
+    }
 }
 
 void CoronaPrivate::init()
@@ -586,7 +589,7 @@ Containment *CoronaPrivate::addContainment(const QString &name, const QVariantLi
     }
 
     // make sure the containments are sorted by id
-    auto position = std::lower_bound(containments.begin(), containments.end(), containment, [](Plasma::Containment *c1, Plasma::Containment *c2) {
+    const auto position = std::lower_bound(containments.begin(), containments.end(), containment, [](Plasma::Containment *c1, Plasma::Containment *c2) {
         return c1->id() < c2->id();
     });
     containments.insert(position, containment);
