@@ -27,7 +27,7 @@ public:
     PopupPlasmaWindowPrivate(PopupPlasmaWindow *_q);
 
     void updateEffectivePopupDirection(const QRect &anchorRect, const QRect &relativePopupPosition);
-    void updateSlideEffect();
+    void updateSlideEffect(const QRect &globalPosition);
     void updatePosition();
     void updatePositionX11(const QPoint &position);
     void updatePositionWayland(const QPoint &position);
@@ -44,6 +44,7 @@ public:
     int m_margin = 0;
     Qt::Edge m_popupDirection = Qt::TopEdge;
     Qt::Edge m_effectivePopupDirection = Qt::TopEdge;
+    Qt::Edges m_nearbyBorders;
 };
 
 PopupPlasmaWindowPrivate::PopupPlasmaWindowPrivate(PopupPlasmaWindow *_q)
@@ -82,9 +83,30 @@ void PopupPlasmaWindowPrivate::updateEffectivePopupDirection(const QRect &anchor
     }
 }
 
-void PopupPlasmaWindowPrivate::updateSlideEffect()
+void PopupPlasmaWindowPrivate::updateSlideEffect(const QRect &globalPosition)
 {
     KWindowEffects::SlideFromLocation slideLocation = KWindowEffects::NoEdge;
+
+    int slideOffset = -1;
+    QScreen *screen = QGuiApplication::screenAt(globalPosition.center());
+    if (screen && m_margin > 0) {
+        const QRect screenGeometry = screen->geometry();
+        switch (m_effectivePopupDirection) {
+        case Qt::TopEdge:
+            slideOffset = screenGeometry.bottom() - globalPosition.bottom() - m_margin;
+            break;
+        case Qt::BottomEdge:
+            slideOffset = globalPosition.top() - screenGeometry.top() - m_margin;
+            break;
+        case Qt::LeftEdge:
+            slideOffset = screenGeometry.right() - globalPosition.right() - m_margin;
+            break;
+        case Qt::RightEdge:
+            slideOffset = globalPosition.left() - screenGeometry.left() - m_margin;
+            break;
+        }
+    }
+
     if (!m_animated) {
         KWindowEffects::slideWindow(q, slideLocation);
         return;
@@ -103,7 +125,7 @@ void PopupPlasmaWindowPrivate::updateSlideEffect()
         slideLocation = KWindowEffects::LeftEdge;
         break;
     }
-    KWindowEffects::slideWindow(q, slideLocation);
+    KWindowEffects::slideWindow(q, slideLocation, slideOffset);
 }
 
 void PopupPlasmaWindowPrivate::updatePosition()
@@ -148,7 +170,7 @@ void PopupPlasmaWindowPrivate::updatePosition()
         relativePopupPosition = relativePopupPosition.translated(-m_visualParent->window()->position());
     }
     updateEffectivePopupDirection(parentAnchorRect.toRect(), relativePopupPosition);
-    updateSlideEffect();
+    updateSlideEffect(popupPosition);
 
     if (KWindowSystem::isPlatformX11()) {
         updatePositionX11(popupPosition.topLeft());
@@ -185,22 +207,17 @@ void PopupPlasmaWindowPrivate::updateBorders(const QRect &globalPosition)
 
     Qt::Edges enabledBorders = Qt::LeftEdge | Qt::RightEdge | Qt::TopEdge | Qt::BottomEdge;
 
-    if (m_margin) {
-        q->setBorders(enabledBorders);
-        return;
-    }
-
     if (m_removeBorderStrategy & PopupPlasmaWindow::AtScreenEdges) {
-        if (globalPosition.top() <= screenGeometry.top()) {
+        if (globalPosition.top() - m_margin <= screenGeometry.top()) {
             enabledBorders.setFlag(Qt::TopEdge, false);
         }
-        if (globalPosition.bottom() >= screenGeometry.bottom()) {
+        if (globalPosition.bottom() + m_margin >= screenGeometry.bottom()) {
             enabledBorders.setFlag(Qt::BottomEdge, false);
         }
-        if (globalPosition.left() <= screenGeometry.left()) {
+        if (globalPosition.left() - m_margin <= screenGeometry.left()) {
             enabledBorders.setFlag(Qt::LeftEdge, false);
         }
-        if (globalPosition.right() >= screenGeometry.right()) {
+        if (globalPosition.right() + m_margin >= screenGeometry.right()) {
             enabledBorders.setFlag(Qt::RightEdge, false);
         }
     }
@@ -221,6 +238,28 @@ void PopupPlasmaWindowPrivate::updateBorders(const QRect &globalPosition)
             break;
         }
     }
+
+    Qt::Edges newNearbyBorders = ~enabledBorders;
+    if (newNearbyBorders.testFlag(Qt::LeftEdge) && newNearbyBorders.testFlag(Qt::RightEdge)) {
+        newNearbyBorders.setFlag(Qt::LeftEdge, false);
+        newNearbyBorders.setFlag(Qt::RightEdge, false);
+    }
+    if (newNearbyBorders.testFlag(Qt::TopEdge) && newNearbyBorders.testFlag(Qt::BottomEdge)) {
+        newNearbyBorders.setFlag(Qt::TopEdge, false);
+        newNearbyBorders.setFlag(Qt::BottomEdge, false);
+    }
+    newNearbyBorders.setFlag(PlasmaQuickPrivate::oppositeEdge(m_effectivePopupDirection), true);
+
+    if (newNearbyBorders != m_nearbyBorders) {
+        m_nearbyBorders = newNearbyBorders;
+        Q_EMIT q->nearbyBordersChanged();
+    }
+
+    if (m_margin) {
+        q->setBorders(Qt::LeftEdge | Qt::RightEdge | Qt::TopEdge | Qt::BottomEdge);
+        return;
+    }
+
     q->setBorders(enabledBorders);
 }
 
@@ -348,6 +387,11 @@ int PopupPlasmaWindow::margin() const
     return d->m_margin;
 }
 
+Qt::Edges PopupPlasmaWindow::nearbyBorders() const
+{
+    return d->m_nearbyBorders;
+}
+
 void PopupPlasmaWindow::setMargin(int margin)
 {
     if (d->m_margin == margin) {
